@@ -1,4 +1,4 @@
-#!/usr/local/bin/python
+#! /usr/bin/env python
 # -*- coding: utf-8 -*-
 
 import unittest
@@ -24,11 +24,17 @@ logger = logging.getLogger(__name__)
 try:
     from PyQt4 import QtGui, QtCore
     from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
-    # from matplotlib.backends.backend_qt4agg import NavigationToolbar2QTAgg as NavigationToolbar
-    from matplotlib.backends.backend_qt4agg import NavigationToolbar2QT as NavigationToolbar
+    try:
+        from matplotlib.backends.backend_qt4agg import NavigationToolbar2QT as NavigationToolbar
+    except:
+        from matplotlib.backends.backend_qt4agg import NavigationToolbar2QTAgg as NavigationToolbar
 except:
-    logger.warning('PyQt4 not detected')
-    pass
+    logger.exception('PyQt4 not detected')
+    print('PyQt4 not detected')
+
+# compatibility between python 2 and 3
+if sys.version_info[0] >= 3:
+    xrange = range
 
 
 class sed3:
@@ -61,25 +67,28 @@ class sed3:
             cmap=matplotlib.cm.Greys_r, seeds=None, contour=None, zaxis=0,
             mouse_button_map={1: 1, 2: 2, 3: 3, 4: 4, 5: 5, 6: 6, 7: 7, 8: 8},
             windowW=None, windowC=None, show=False, sed3_on_close=None, figure=None,
-            show_axis=False
+            show_axis=False, flipV=False, flipH=False
     ):
         """
 
         :param img:
-        :param voxelsizemm:
+        :param voxelsize: size of voxel
         :param initslice:
         :param colorbar:
         :param cmap:
-        :param seeds:
-        :param contour:
+        :param seeds: define seeds
+        :param contour: contour
         :param zaxis:
         :param mouse_button_map:
-        :param windowW:
-        :param windowC:
+        :param windowW: window width
+        :param windowC: window center
         :param show:
         :param sed3_on_close:
         :param figure:
         :param show_axis:
+        :param flipH: horizontal flip
+        :param flipV: vertical flip
+
         :return:
         """
 
@@ -117,6 +126,8 @@ class sed3:
         self.colorbar = colorbar
         self.cmap = cmap
         self.show_axis = show_axis
+        self.flipH = flipH
+        self.flipV = flipV
         if seeds is None:
             self.seeds = np.zeros(self.imgshape, np.int8)
         else:
@@ -334,7 +345,7 @@ class sed3:
                 # data = np.transpose(data, (0, 1, 2))
                 pass
             else:
-                print "problem with zaxis in _rotate_start()"
+                print("problem with zaxis in _rotate_start()")
                 logger.warning("problem with zaxis in _rotate_start()")
 
         return data
@@ -357,11 +368,11 @@ class sed3:
                 elif zaxis == 2:
                     pass
                 else:
-                    print "problem with zaxis in _rotate_start()"
+                    print("problem with zaxis in _rotate_start()")
                     logger.warning("problem with zaxis in _rotate_start()")
 
             else:
-                print "There is a danger in calling show() twice"
+                print("There is a danger in calling show() twice")
                 logger.warning("There is a danger in calling show() twice")
 
         return data
@@ -372,14 +383,31 @@ class sed3:
 
         self.draw_slice()
 
+    def __flip(self, sliceimg):
+        """
+        Flip if asked in self.flipV or self.flipH
+        :param sliceimg: one image slice
+        :return: flipp
+        """
+        if self.flipH:
+            sliceimg = sliceimg[:, -1:0:-1]
+
+        if self.flipV:
+            sliceimg = sliceimg [-1:0:-1,:]
+
+        return sliceimg
+
     def draw_slice(self):
         sliceimg = self.img[:, :, int(self.actual_slice)]
+        sliceimg = self.__flip(sliceimg)
         self.imsh = self.ax.imshow(sliceimg, self.cmap, vmin=self.imgmin,
                                    vmax=self.imgmax, interpolation='nearest')
         # plt.hold(True)
         # pdb.set_trace();
+        sliceseeds = self.seeds[:, :, int(self.actual_slice)]
+        sliceseeds = self.__flip(sliceseeds)
         self.ax.imshow(self.prepare_overlay(
-            self.seeds[:, :, int(self.actual_slice)]
+            sliceseeds
         ), interpolation='nearest', vmin=self.imgmin, vmax=self.imgmax)
 
         # vykreslení okraje
@@ -389,8 +417,10 @@ class sed3:
             try:
                 # exception catch problem with none object in image
                 # ctr =
+                slicecontour = self.contour[:, :, int(self.actual_slice)]
+                slicecontour = self.__flip(slicecontour)
                 self.ax.contour(
-                    self.contour[:, :, int(self.actual_slice)], 1,
+                    slicecontour, 1,
                     levels=[0.5, 1.5, 2.5],
                     linewidths=2)
             except:
@@ -400,7 +430,7 @@ class sed3:
         # self.ax.set_axis_below(False)
         self._ticklabels()
 
-        # print ctr
+        # print(ctr)
         # import pdb; pdb.set_trace()
 
         self.fig.canvas.draw()
@@ -493,7 +523,7 @@ class sed3:
         self.actual_slice_slider.set_val(self.actual_slice)
         # tim, ze dojde ke zmene slideru je show_slce volan z nej
         # self.show_slice()
-        # print self.actual_slice
+        # print(self.actual_slice)
 
     # malování -------------------
     def on_press(self, event):
@@ -502,7 +532,7 @@ class sed3:
             return
         # contains, attrd = self.rect.contains(event)
         # if not contains: return
-        # print 'event contains', self.rect.xy
+        # print('event contains', self.rect.xy)
         # x0, y0 = self.rect.xy
         self.press = [event.xdata], [event.ydata], event.button
         # self.press1 = True
@@ -514,7 +544,7 @@ class sed3:
 
         if event.inaxes != self.ax:
             return
-        # print event.inaxes
+        # print(event.inaxes)
 
         x0, y0, btn = self.press
         x0.append(event.xdata)
@@ -524,7 +554,7 @@ class sed3:
         'on release we reset the press data'
         if self.press is None:
             return
-        # print self.press
+        # print(self.press)
         x0, y0, btn = self.press
         if btn == 1:
             color = 'r'
@@ -574,8 +604,12 @@ class sed3:
         return self.img[self.seeds == label]
 
 
-def show_slices(data3d, contour=None, seeds=None, axis=0, slice_step=1,
-                shape=None, show=True, flipH=False, flipV=False):
+def show_slices(data3d, contour=None, seeds=None, axis=0, slice_step=None,
+                shape=None, show=True,
+                flipH=False, flipV=False,
+                first_slice_offset=0,
+                first_slice_offset_to_see_seed_with_label=None
+                ):
     """
     Show slices as tiled image
 
@@ -583,12 +617,37 @@ def show_slices(data3d, contour=None, seeds=None, axis=0, slice_step=1,
     :param contour: Data for contouring
     :param seeds: Seed data
     :param axis: Axis for sliceing
-    :param slice_step: Show each "slice_step"-th slice
+    :param slice_step: Show each "slice_step"-th slice, can be float
+    :param shape: set shape of output tiled image. slice_step is estimated if it is not set explicitly
+    :param first_slice_offset: set offset of first slice
+    :param first_slice_offset_to_see_seed_with_label: find offset to see slice with seed with defined label
     """
 
-    data3d = _import_data(data3d, axis=axis, slice_step=slice_step)
-    contour = _import_data(contour, axis=axis, slice_step=slice_step)
-    seeds = _import_data(seeds, axis=axis, slice_step=slice_step)
+    # odhad slice_step, neni li zadan
+    # slice_step estimation
+    # TODO make precise estimation (use np.linspace to indexing?)
+    if slice_step is None:
+        if shape is None:
+            slice_step = 1
+        else:
+            slice_step = ((data3d.shape[axis] - first_slice_offset ) / float(np.prod(shape)))
+
+
+
+    if first_slice_offset_to_see_seed_with_label is not None:
+        if seeds is not None:
+            inds = np.nonzero(seeds==first_slice_offset_to_see_seed_with_label)
+            # print(inds)
+            # take first one with defined seed
+            # ind = inds[axis][0]
+            # take most used index
+            ind = np.median(inds[axis])
+            first_slice_offset = ind % slice_step
+
+
+    data3d = _import_data(data3d, axis=axis, slice_step=slice_step, first_slice_offset=first_slice_offset)
+    contour = _import_data(contour, axis=axis, slice_step=slice_step, first_slice_offset=first_slice_offset)
+    seeds = _import_data(seeds, axis=axis, slice_step=slice_step, first_slice_offset=first_slice_offset)
 
     number_of_slices = data3d.shape[axis]
     # square image
@@ -597,14 +656,14 @@ def show_slices(data3d, contour=None, seeds=None, axis=0, slice_step=1,
     #     sh = [nn, nn]
 
     # 4:3 image
-    sh = shape
-    if sh is None:
+    meta_shape = shape
+    if meta_shape is None:
         na = int(math.ceil(number_of_slices * 16.0 / 9.0) ** 0.5)
         nb = int(math.ceil(float(number_of_slices) / na))
-        sh = [nb, na]
+        meta_shape = [nb, na]
 
     dsh = __get_slice(data3d, 0, axis).shape
-    slimsh = [int(dsh[0] * sh[0]), int(dsh[1] * sh[1])]
+    slimsh = [int(dsh[0] * meta_shape[0]), int(dsh[1] * meta_shape[1])]
     slim = np.zeros(slimsh, dtype=data3d.dtype)
     slco = None
     slse = None
@@ -618,20 +677,20 @@ def show_slices(data3d, contour=None, seeds=None, axis=0, slice_step=1,
     for i in range(0, number_of_slices):
         cont = None
         seeds2d = None
-        im2d = __get_slice(data3d, i, axis)
+        im2d = __get_slice(data3d, i, axis, flipH=flipH, flipV=flipV)
         if contour is not None:
-            cont = __get_slice(contour, i, axis)
-            slco = __put_slice_in_slim(slco, cont, sh, i)
+            cont = __get_slice(contour, i, axis, flipH=flipH, flipV=flipV)
+            slco = __put_slice_in_slim(slco, cont, meta_shape, i)
         if seeds is not None:
-            seeds2d = __get_slice(seeds, i, axis)
-            slse = __put_slice_in_slim(slse, seeds2d, sh, i)
+            seeds2d = __get_slice(seeds, i, axis, flipH=flipH, flipV=flipV)
+            slse = __put_slice_in_slim(slse, seeds2d, meta_shape, i)
         #         plt.axis('off')
         #         plt.subplot(sh[0], sh[1], i+1)
         #         plt.subplots_adjust(wspace=0, hspace=0)
 
-        slim = __put_slice_in_slim(slim, im2d, sh, i)
+        slim = __put_slice_in_slim(slim, im2d, meta_shape, i)
     #         show_slice(im2d, cont, seeds2d)
-    show_slice(slim, slco, slse, flipH=flipH, flipV=flipV)
+    show_slice(slim, slco, slse)
     if show:
         plt.show()
 
@@ -641,18 +700,35 @@ def show_slices(data3d, contour=None, seeds=None, axis=0, slice_step=1,
 #     pass
 
 
-def __get_slice(data, slice_number, axis=0):
+def __get_slice(data, slice_number, axis=0, flipH=False, flipV=False):
+    """
+
+    :param data:
+    :param slice_number:
+    :param axis:
+    :param flipV: vertical flip
+    :param flipH: horizontal flip
+    :return:
+    """
     if axis == 0:
-        return data[slice_number, :, :]
+        data2d =  data[slice_number, :, :]
     elif axis == 1:
-        return data[:, slice_number, :]
+        data2d = data[:, slice_number, :]
     elif axis == 2:
-        return data[:, :, slice_number]
+        data2d = data[:, :, slice_number]
     else:
         logger.error("axis number error")
-        print "axis number error"
+        print("axis number error")
         return None
 
+    if flipV:
+        if data2d is not None:
+            data2d = data2d[-1:0:-1,:]
+    if flipH:
+        if data2d is not None:
+            data2d = data2d[:, -1:0:-1]
+    return data2d
+
 
 def __put_slice_in_slim(slim, dataim, sh, i):
     """
@@ -680,35 +756,21 @@ def __put_slice_in_slim(slim, dataim, sh, i):
 # def close():
 #     plt.close()
 
+def sigmoid(x, x0, k):
+     y = 1 / (1 + np.exp(-k*(x-x0)))
+     return y
 
-def show_slice(data2d, contour2d=None, seeds2d=None, flipV=False, flipH=False):
+def show_slice(data2d, contour2d=None, seeds2d=None):
     """
 
     :param data2d:
     :param contour2d:
     :param seeds2d:
-    :param flipV: vertical flip
-    :param flipH: horizontal flip
     :return:
     """
+
     import copy as cp
     # Show results
-    if flipV:
-
-        if data2d is not None:
-            data2d = data2d[-1:0:-1,:]
-        if seeds2d is not None:
-            seeds2d = seeds2d[-1:0:-1,:]
-        if contour2d is not None:
-            contour2d= contour2d[-1:0:-1,:]
-    if flipH:
-        if data2d is not None:
-            data2d = data2d[:, -1:0:-1]
-        if seeds2d is not None:
-            seeds2d = seeds2d[:, -1:0:-1]
-        if contour2d is not None:
-            contour2d= contour2d[:, -1:0:-1]
-        # data2d = data2d[:, ::-1]
 
     colormap = cp.copy(plt.cm.get_cmap('brg'))
     colormap._init()
@@ -719,28 +781,63 @@ def show_slice(data2d, contour2d=None, seeds2d=None, flipV=False, flipH=False):
         plt.contour(contour2d, levels=[0.5, 1.5, 2.5])
     if seeds2d is not None:
         # Show results
-        colormap = copy.copy(plt.cm.get_cmap('brg'))
+        colormap = copy.copy(plt.cm.get_cmap('Paired'))
+        # colormap = copy.copy(plt.cm.get_cmap('gist_rainbow'))
         colormap._init()
+
         colormap._lut[0, 3] = 0
 
+        tmp0 = copy.copy(colormap._lut[:,0])
+        tmp1 = copy.copy(colormap._lut[:,1])
+        tmp2 = copy.copy(colormap._lut[:,2])
+
+        colormap._lut[:, 0] = sigmoid(tmp0, 0.5, 5)
+        colormap._lut[:, 1] = sigmoid(tmp1, 0.5, 5)
+        colormap._lut[:, 2] = 0# sigmoid(tmp2, 0.5, 5)
+        # seed 4
+        colormap._lut[140:220:, 1] = 0.7# sigmoid(tmp2, 0.5, 5)
+        colormap._lut[140:220:, 0] = 0.2# sigmoid(tmp2, 0.5, 5)
+        # seed 2
+        colormap._lut[40:120:, 1] = 1.# sigmoid(tmp2, 0.5, 5)
+        colormap._lut[40:120:, 0] = 0.1# sigmoid(tmp2, 0.5, 5)
+
+
+        # seed 2
+        colormap._lut[120:150:, 0] = 1.# sigmoid(tmp2, 0.5, 5)
+        colormap._lut[120:150:, 1] = 0.1# sigmoid(tmp2, 0.5, 5)
+
+        # my colors
+
+        # colormap._lut[1,:] = [.0,.1,.0,1]
+        # colormap._lut[2,:] = [.1,.1,.0,1]
+        # colormap._lut[3,:] = [.1,.1,.1,1]
+        # colormap._lut[4,:] = [.3,.3,.3,1]
+
         plt.imshow(seeds2d, cmap=colormap, interpolation='none')
 
 
-def __select_slices(data, axis, slice_step):
+def __select_slices(data, axis, slice_step, first_slice_offset=0):
     if data is None:
         return None
 
+    inds = np.floor(np.arange(first_slice_offset, data.shape[axis], slice_step)).astype(np.int)
+    # import ipdb
+    # ipdb.set_trace()
+    # logger.warning("select slices")
 
     if axis == 0:
-        data = data[::slice_step, :, :]
+        # data = data[first_slice_offset::slice_step, :, :]
+        data = data[inds, :, :]
     if axis == 1:
-        data = data[:, ::slice_step, :]
+        # data = data[:, first_slice_offset::slice_step, :]
+        data = data[:, inds, :]
     if axis == 2:
-        data = data[:, :, ::slice_step]
+        # data = data[:, :, first_slice_offset::slice_step]
+        data = data[:, :, inds]
     return data
 
 
-def _import_data(data, axis, slice_step):
+def _import_data(data, axis, slice_step, first_slice_offset=0):
     """
     import ndarray or SimpleITK data
     """
@@ -751,54 +848,97 @@ def _import_data(data, axis, slice_step):
     except:
         pass
 
-    data = __select_slices(data, axis, slice_step)
+    data = __select_slices(data, axis, slice_step, first_slice_offset=first_slice_offset)
     return data
 
 
 # self.rect.figure.canvas.draw()
 
 # return data
+try:
+    from PyQt4 import QtGui, QtCore
+    class sed3qt(QtGui.QDialog):
+        def __init__(self, *pars, **params):
+            # def __init__(self,parent=None):
+            parent = None
 
-class sed3qt(QtGui.QDialog):
-    def __init__(self, *pars, **params):
-        # def __init__(self,parent=None):
-        parent = None
+            QtGui.QDialog.__init__(self, parent)
+            # super(Window, self).__init__(parent)
+            # self.setupUi(self)
+            self.figure = plt.figure()
+            self.canvas = FigureCanvas(self.figure)
+            self.toolbar = NavigationToolbar(self.canvas, self)
 
-        QtGui.QDialog.__init__(self, parent)
-        # super(Window, self).__init__(parent)
-        # self.setupUi(self)
-        self.figure = plt.figure()
-        self.canvas = FigureCanvas(self.figure)
-        self.toolbar = NavigationToolbar(self.canvas, self)
+            # set the layout
+            layout = QtGui.QVBoxLayout()
+            layout.addWidget(self.toolbar)
+            layout.addWidget(self.canvas)
+            # layout.addWidget(self.button)
+            self.setLayout(layout)
 
-        # set the layout
-        layout = QtGui.QVBoxLayout()
-        layout.addWidget(self.toolbar)
-        layout.addWidget(self.canvas)
-        # layout.addWidget(self.button)
-        self.setLayout(layout)
+            # def set_params(self, *pars, **params):
+            # import sed3.sed3
 
-        # def set_params(self, *pars, **params):
-        # import sed3.sed3
+            params["figure"] = self.figure
+            self.sed = sed3(*pars, **params)
+            self.sed.sed3_on_close = self.callback_close
+            # ed.show()
+            self.output = None
 
-        params["figure"] = self.figure
-        self.sed = sed3(*pars, **params)
-        self.sed.sed3_on_close = self.callback_close
-        # ed.show()
-        self.output = None
+        def callback_close(self, sed):
+            self.output = sed
+            sed.prepare_output_data()
+            self.seeds = sed.seeds
+            self.close()
 
-    def callback_close(self, sed):
-        self.output = sed
-        sed.prepare_output_data()
-        self.seeds = sed.seeds
-        self.close()
+        def show(self):
+            return self.sed.show_fcn()
 
-    def show(self):
-        return self.sed.show_fcn()
+        def get_values(self):
+            return self.sed
 
-    def get_values(self):
-        return self.sed
+    class sed3qtWidget(QtGui.QWidget):
+        def __init__(self, *pars, **params):
+            # def __init__(self,parent=None):
+            parent = None
 
+            # QtGui.QWidget.__init__(self, parent)
+            super(sed3qtWidget, self).__init__(parent)
+            # self.setupUi(self)
+            self.figure = plt.figure()
+            self.canvas = FigureCanvas(self.figure)
+            # self.toolbar = NavigationToolbar(self.canvas, self)
+
+            # set the layout
+            layout = QtGui.QVBoxLayout()
+            # layout.addWidget(self.toolbar)
+            layout.addWidget(self.canvas)
+            # layout.addWidget(self.button)
+            self.setLayout(layout)
+
+            # def set_params(self, *pars, **params):
+            # import sed3.sed3
+
+            params["figure"] = self.figure
+            self.sed = sed3(*pars, **params)
+            self.sed.sed3_on_close = self.callback_close
+            # ed.show()
+            self.output = None
+
+        def callback_close(self, sed):
+            self.output = sed
+            sed.prepare_output_data()
+            self.seeds = sed.seeds
+            self.close()
+
+        def show(self):
+            super(sed3qtWidget, self).show()
+            return self.sed.show_fcn()
+
+        def get_values(self):
+            return self.sed
+except:
+    pass
 
 # --------------------------tests-----------------------------
 class Tests(unittest.TestCase):
@@ -996,7 +1136,7 @@ def slices(img, shape=[3, 4]):
     for i in range(0, i_max):
         # i = 0
         islice = round((img.shape[0] / float(i_max)) * i)
-        #         print islice
+        #         print(islice)
         imgi = img[islice, :, :]
         coords = index_to_coords(i, sh)
         aic = np.asarray(img.shape[-2:]) * coords
@@ -1004,8 +1144,8 @@ def slices(img, shape=[3, 4]):
         allimg[aic[0]:aic[0] + imgi.shape[-2], aic[1]:aic[1] + imgi.shape[-1]] = imgi
 
     #     plt.imshow(imgi)
-    #     print imgi.shape
-    #     print img.shape
+    #     print(imgi.shape)
+    #     print(img.shape)
     return allimg
 
 
